<?php

namespace Drupal\revision_graph;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Path\CurrentPathStack;

class RevisionGraphStorage {

  const UNTRACKED_BANCH_NAME = 'untracked';

  /**
   * The default database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The current path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * Creates a RevisionGraphStorage object.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The default database connection.
   * @param \Drupal\Core\Path\CurrentPathStack $current_path
   *   The current path.
   */
  public function __construct(Connection $connection, CurrentPathStack $current_path) {
    $this->connection = $connection;
    $this->currentPath = $current_path;
  }

  public function initGraph(ContentEntityInterface $entity) {
    $this->connection->insert('revision_graph')->fields([
      'primary_id' => $entity->id(),
      'version_id' => $entity->getRevisionId(),
      'parent_version_id' => $entity->getRevisionId(),
      'type' => $entity->getEntityTypeId(),
      'branch' => $this->formatBranchName($entity->getRevisionId(), $entity->language()->getId()),
    ])->execute();
  }

  public function updateGraph(ContentEntityInterface $entity) {
    // @TODO hunt actual parent version id.
    $parent_version_id = $entity->original->getRevisionId();
    $branch = $this->getParentBranchName($parent_version_id);

    // For some reason we have default revision id in original.
    // Hackish way to sniff the actual parent.
    // @TODO Check if this is a bug in core due to some optimisation.
    $current_path = $this->currentPath->getPath();
    $path_args = explode('/', $current_path);
    if ($path_args[1] == 'node' && $path_args[3] == 'revisions' && $path_args[5] == 'revert') {
      $parent_version_id = (int) $path_args[4];
      $branch = $parent_version_id;
    }

    $result = $this->connection->query("SELECT * FROM {revision_graph} WHERE primary_id = :primary_id AND version_id = :version_id AND type = :type", [
      ':primary_id' => $entity->id(),
      ':version_id' => $entity->getRevisionId(),
      ':type' => $entity->getEntityTypeId(),
    ])->fetchAll();
    if (empty($result)) {
      $this->connection->insert('revision_graph')->fields([
        'primary_id' => $entity->id(),
        'version_id' => $entity->getRevisionId(),
        'parent_version_id' => $parent_version_id,
        'type' => $entity->getEntityTypeId(),
        'branch' => $this->formatBranchName($branch, $entity->language()->getId()),
      ])->execute();
    }
  }

  protected function formatBranchName($parent_branch, $language_id): string {
    [$branch] = explode(':', $parent_branch);
    return implode(':', [$branch, $language_id]);
  }

  protected function getParentBranchName($version_id): string {
    $result = $this->connection->query("SELECT * FROM {revision_graph} WHERE version_id = :vid", [':vid' => $version_id])->fetchAll();
    $branch_name = self::UNTRACKED_BANCH_NAME;
    foreach ($result as $r) {
      $branch_name = $r->branch;
      break;
    }
    return $branch_name;
  }

}
